"""
债券交易指标
"""
import pandas as pd
from datetime import datetime
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB
from WindPy import w

w.start()


class BondTrading:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_bond_trading'

    def get_bond_trading_data(self):
        """
        R001成交量、R007成交量、质押式回购成交量
        """
        index_list = ['M0330244', 'M0330245', 'M0041739']
        name_dict = {"M0330244": "R001", "M0330245": "R007", "M0041739": "bank_repo"}
        res = w.edb(','.join(index_list), self.start_date, self.end_date)
        if res.ErrorCode != 0:
            data = pd.DataFrame()
            print("fetch bond trading data error: start_date = {}, end_date = {}".format(self.start_date, self.end_date))
        else:
            if len(res.Data) == 1:
                data = pd.DataFrame(res.Data[0], index=res.Codes, columns=res.Times).T
            else:
                data = pd.DataFrame(res.Data, index=res.Codes, columns=res.Times).T
            data.index.name = 'trade_date'
            data.reset_index(inplace=True)
            data['trade_date'] = data['trade_date'].apply(lambda x: datetime.strftime(x, '%Y%m%d'))
            data.rename(columns=name_dict, inplace=True)

        return data

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_bond_trading_data()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table mac_bond_trading(
                id int auto_increment primary key,
                trade_date date not null unique,
                R001 decimal(8, 2),
                R007 decimal(8, 2),
                bank_repo decimal(8, 2))
            """
            create_table(self.table_name, sql_script)
            data = self.get_bond_trading_data()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    BondTrading('2010-01-01', '2021-06-25', is_increment=0).get_construct_result()


# -*- coding: utf-8 -*-

from openpyxl import load_workbook
from pathlib import Path
import configparser
import io
import os
import pandas as pd
import pickle
import zipfile


class FileWriter:
    @staticmethod
    def get_excel_writer(file_name, append=True):
        try:
            if not append:
                writer = pd.ExcelWriter(file_name, engine='openpyxl')
                return writer
            book = load_workbook(file_name)  # existed
            writer = pd.ExcelWriter(file_name, engine='openpyxl')
            writer.book = book
            writer.sheets = dict((ws.title, ws) for ws in book.worksheets)
        except:
            writer = pd.ExcelWriter(file_name, engine='openpyxl')
        return writer

    @staticmethod
    def write_file_content(file_name, content):
        with io.open(file_name, "a", encoding="utf-8") as f:
            f.write(content)
        return

    @staticmethod
    def write_file_property(filename, section_name, key, value):
        config = configparser.ConfigParser()
        config.read(filename, encoding="utf-8")
        if section_name not in config:
            config.add_section(section_name)
        config.set(section_name, key, value)
        with io.open(filename, 'w', encoding="utf-8") as configfile:
            config.write(configfile)

    @staticmethod
    def clear_file(file_name):
        io.open(file_name, 'w', encoding="utf-8").close()
        return

    @staticmethod
    def df_to_hdf(df, file_name):
        if Path(file_name).is_file():
            os.remove(file_name)
        df.to_hdf(file_name, 'table')
        return

    @staticmethod
    def object_dump_to_file(obj, file_name):
        if Path(file_name).is_file():
            os.remove(file_name)
        pickle.dump(obj, open(file_name, "wb"), protocol=2)
        return

    @staticmethod
    def zip_direction_file(path_name, file_name):
        zip_file = zipfile.ZipFile(file_name, 'w', zipfile.ZIP_DEFLATED)
        for root, dirs, files in os.walk(path_name):
            for file in files:
                zip_file.write(os.path.join(root, file), file, zipfile.ZIP_DEFLATED)
        zip_file.close()
        return


"""
股票市场集中度指标
"""
import pandas as pd
import requests
import json
from hbshare.quant.Kevin.asset_allocation.macro_index.util import create_table, delete_duplicate_records, WriteToDB


class TradingCRNCalculator:
    def __init__(self, start_date, end_date, is_increment=1):
        self.start_date = start_date
        self.end_date = end_date
        self.is_increment = is_increment
        self.table_name = 'mac_stock_trading_cr'
        self._init_api_params()

    def _init_api_params(self):
        self.url = 'http://fdc-query.intelnal.howbuy.com/query/data/commonapi?dataTrack=xxxxx'
        self.headers = {'Content-Type': 'application/json'}
        self.post_body = {"database": "readonly", "sql": None}

    def get_stock_trading_cr(self):
        # calendar
        sql_script = "SELECT JYRQ, SFJJ, SFZM, SFYM FROM funddb.JYRL WHERE JYRQ >= {} and JYRQ <= {}".format(
            self.start_date.replace('-', ''), self.end_date.replace('-', ''))
        post_body = self.post_body
        post_body['sql'] = sql_script
        res = requests.post(url=self.url, data=json.dumps(post_body), headers=self.headers).json()
        df = pd.DataFrame(res['data']).rename(
            columns={"JYRQ": 'calendarDate', "SFJJ": 'isOpen',
                     "SFZM": "isWeekEnd", "SFYM": "isMonthEnd"}).sort_values(by='calendarDate')
        df['isOpen'] = df['isOpen'].astype(int).replace({0: 1, 1: 0})
        df['isWeekEnd'] = df['isWeekEnd'].fillna(0).astype(int)
        df['isMonthEnd'] = df['isMonthEnd'].fillna(0).astype(int)
        trading_day_list = df[df['isOpen'] == 1]['calendarDate'].tolist()
        # calculate
        cr5_list, cr10_list, avg_mkt_list = [], [], []
        from tqdm import tqdm
        for date in tqdm(trading_day_list):
            sql_script = "SELECT SYMBOL, TDATE, VATURNOVER, TCAP FROM finchina.CHDQUOTE WHERE TDATE = {}".format(date)
            post_body = self.post_body
            post_body['sql'] = sql_script
            res = requests.post(url=self.url, data=json.dumps(post_body), headers=self.headers).json()
            df = pd.DataFrame(res['data'])
            df = df[df['SYMBOL'].str[0].isin(['0', '3', '6'])]
            df.rename(columns={"SYMBOL": "ticker", "TDATE": "tradeDate", "TCAP": "marketValue",
                               "VATURNOVER": "turnoverValue"}, inplace=True)
            # 剔除停牌、没有成交额的数据
            df = df[df['turnoverValue'] > 0.]
            quantile_5, quantile_10 = df['turnoverValue'].quantile(0.95), df['turnoverValue'].quantile(0.90)
            cr5 = df[df['turnoverValue'] >= quantile_5]['turnoverValue'].sum() / df['turnoverValue'].sum()
            cr10 = df[df['turnoverValue'] >= quantile_10]['turnoverValue'].sum() / df['turnoverValue'].sum()
            cr5_list.append(cr5)
            cr10_list.append(cr10)

            avg_mkt = (df['turnoverValue'] * df['marketValue']).sum() / df['turnoverValue'].sum() / 1e+8
            avg_mkt_list.append(avg_mkt)

        results = pd.DataFrame(
            {"trade_date": trading_day_list, "cr5": cr5_list, "cr10": cr10_list, "avg_mkt": avg_mkt_list})
        # 上证综指走势
        sql_script = "SELECT JYRQ, SPJG FROM funddb.ZSJY WHERE ZQDM = '{}' and JYRQ >= {} and JYRQ <= {}".format(
            '000001', trading_day_list[0], trading_day_list[-1])
        post_body = self.post_body
        post_body['sql'] = sql_script
        res = requests.post(url=self.url, data=json.dumps(post_body), headers=self.headers).json()
        df = pd.DataFrame(res['data']).rename(columns={"JYRQ": "trade_date", "SPJG": "close"})[['trade_date', 'close']]

        df = pd.merge(results, df, on='trade_date')

        return df

    def get_construct_result(self):
        if self.is_increment == 1:
            data = self.get_stock_trading_cr()
            sql_script = "delete from {} where trade_date in ({})".format(
                self.table_name, ','.join(data['trade_date'].tolist()))
            # delete first
            delete_duplicate_records(sql_script)
            # add new records
            WriteToDB().write_to_db(data, self.table_name)
        else:
            sql_script = """
                create table mac_stock_trading_cr(
                id int auto_increment primary key,
                trade_date date not null unique,
                cr5 decimal(5, 4),
                cr10 decimal(5, 4),
                avg_mkt decimal(7, 2),
                close decimal(7, 2)) 
            """
            create_table(self.table_name, sql_script)
            data = self.get_stock_trading_cr()
            WriteToDB().write_to_db(data, self.table_name)


if __name__ == '__main__':
    TradingCRNCalculator('2022-04-01', '2022-07-21', is_increment=1).get_construct_result()
    # TradingCRNCalculator('2019-01-02', '2019-01-12', is_increment=1).get_construct_result()